ActiveAdmin.register Message do
  menu :label => "Collections"
  
  config.batch_actions = false
  
  scope :all, default: true
  scope :public_ones
  scope :private_ones
  scope :featured_ones
  scope :show_in_explore_page

  actions :all, :except => [:new]

  member_action :clone, :method => :get do
    message = Message.find( params[:id] )
    
    if Message.featured_ones.active.count < 3
      message.clone_public.save!
      
      flash[:notice] = 'New Collection has been cloned successfully but it will take some time to transfer items to it and it will be activated by then!'
      redirect_to :back
    else
      flash[:error] = 'You reached max limit. Remove or Deactivate some public collections to be able to perform action!'
      redirect_to :back
    end
  end

  index do
    column :id
    column :subject
    
    column 'Active' do |record|
      record.active?
    end
    column 'Public' do |record|
      record.public?
    end
    column 'Owner ID', :owner_id
    column :items_count do |record|
      record.items.count
    end
    column :impressions_count do |record|
      record.cached_impressions_count
    end
    
    actions defaults: true do |message|
      link_to 'Clone Public', clone_admin_message_path(message)
    end
  end
  
  show do |record|
    attributes_table do
      row :id
      row :owner
      row :subject
      row :url
      row :active
      row :public
      row :items_count
      
      row :add_product_enabled
      row :add_comment_enabled
      row :hover_over_enabled
      
      row :show_in_explore_page
      
      row :impressions_count do
        record.cached_impressions_count
      end
      row :site_impressions_count do
        record.cached_site_impressions_count
      end
      row :widget_impressions_count do
        record.cached_widget_impressions_count
      end
      
      if record.image_uploaded? && record.image.file?
        row :image do
          image_tag(record.image.url(:size300))
        end
      end
    end
  end
  
  form do |f|
    f.inputs "Options" do
      f.input :add_product_enabled
      f.input :add_comment_enabled
      f.input :hover_over_enabled
      f.input :show_in_explore_page
      
      f.input :active
    end
    
    f.inputs "Info" do
      f.input :image

      if f.object.image.file?
        f.input :remove_image, as: :boolean, required: false, label: "Remove Image"
      end
      
      f.input :url
    end
    
    f.actions
  end
end