ActiveAdmin.register User do
  scope :all, default: true
  scope :invitation_not_accepted

  index do
    selectable_column
    column :id
    column :first_name
    column :last_name
    column :email
    column :gender
    column :age_range
    column :invited_by_id
    column "Created Collections" do |user|
      user.owned_messages.count
    end
    column "Total Collections" do |user|
      user.total_messages(true).count
    end    
    default_actions
  end

  csv do
    column :id
    column :first_name
    column :last_name
    column :email
    column :gender
    column :age_range
    column :invited_by_id
    column "Created Collections" do |user|
      user.owned_messages.count
    end
    column "Total Collections" do |user|
      user.total_messages.count
    end
    column :notification_frequency
  end

  batch_action :send_invitation_to do |selection|
    if User.find(selection).each(&:invite!)
      redirect_to collection_path, notice: 'Selected users were sent an invitation message'
    else
      redirect_to collection_path, alert: 'Failed sending invitation message to the selected users'
    end
  end
  
  batch_action :send_digest_email do |selection|
    if User.find(selection).each {|u| Mailer.daily_digest_mail(u).deliver }
      redirect_to collection_path, notice: 'Selected users were sent the daily digest email'
    else
      redirect_to collection_path, alert: 'Failed sending daily digest email'
    end
  end
end