Tisket.require('Tisket.Message', function(){
  var that = {};
  
  var messageId = null;
  var currentUserId = null;
  var additionThreshold = 5000;
  var temporalItems = {};
  
  var onDomLoad = function(options){
    extractOptions(options);
    
    addOrRemoveChromeExtensionAd();
    adjustDomElementsPositions();
    
    initDragAndDrop();
  };
  that.onDomLoad = onDomLoad;
  
  var onWindowLoad = function(){
    initItemsHeights();
    
    initRecipients();
    
    Tisket.SideNavigation.init({
      currentUserId: currentUserId
    });
    
    initNewItemTrigger();
    
    initItems();
    
    initNewCommentForm();
  };
  that.onWindowLoad = onWindowLoad;
  
  var initItemsHeights = function(){
    if( Tisket.Layout.isMobile() )
      return;
    
    if( Tisket.Layout.isTablet() )
      var itemsPerRow = 2;
    else
      var itemsPerRow = 3;
      
    var allItems = $('.js-item');
    
    allItems.height('auto');
    
    var start = 0;
    while(true){
      var end = (start + itemsPerRow);
      
      var items = allItems.slice(start, end);
      if( items.length == 0 )
        break;
      
      var maxHeight = 0;
      items.each(function(){
        var item = $(this);
        
        if( Tisket.Layout.containerFullHeight(item) > maxHeight )
          maxHeight = Tisket.Layout.containerFullHeight(item);
      });
      items.each(function(){
        var item = $(this);
        
        item.height( maxHeight );
      });
      
      start = end;
    }
  };
  that.initItemsHeights = initItemsHeights;
  
  var addOrRemoveChromeExtensionAd = function(){
    if( $('body').attr('data-tisket-extension-installed') == '1' ){
      $('.js-chromeExtension').remove();
    }
  };
  
  var adjustDomElementsPositions = function(){
    if( !Tisket.Layout.isMobile() )
      return;
    
    $('.js-mobileCollectionInfo').html( $('.js-desktopCollectionInfo').html() );
    $('.js-desktopCollectionInfo').html('');
    
    var desktopItems = $('.js-desktopItems');
    var mobileItems = $('.js-mobileItems');  
    $('.js-desktopItems .js-item').each(function(){
      var item = $(this);
      
      item.find('.js-imagesCarousel').remove();
      mobileItems.append( item.wrap('<li>').parent() );
    });
    desktopItems.html('');
  };
  
  var initDragAndDrop = function(){
    if( Tisket.Layout.isMobile() || Tisket.Layout.isTablet() )
      return;
    
    $(".js-collection").droppable({
      accept: ".js-item",
      hoverClass: "hoverEffect",
      tolerance: "touch",
      drop: function(event, ui){
        var draggedItem = ui.draggable;
        
        var collection = $(this);
        
        collection.addClass("activeItem");
        
        $.ajax({
          url: ('/items/' + draggedItem.attr('data-itemId') + '/copy_or_move.js'),
          type: 'post',
          dataType: 'json',
          data: {
            authenticity_token: Tisket.Layout.authenticityToken,
            collection_id: collection.attr('data-collectionId')
          },
          success: function(response){
            if( response.applied_action == 'move' ){
              draggedItem.remove();
              Tisket.Layout.adjustFooterPosition();
            }
          },
          error: function(jqXHR){
            try{
              var response = JSON.parse(jqXHR.responseText);
            }
            catch(e){
              Tisket.Notifications.showErrorMessage('Something went wrong!');
            }
            
            Tisket.Notifications.showErrorMessage(response.error_message);
          }
        });
      },
      over: function(){
        var collection = $(this);
        
        collection.removeClass("activeItem");
      }
    });
  };
  
  var initItems = function(){
    $('.js-items .js-item').each(function(){
      initItem( $(this) );
    });
    
    if( Tisket.Layout.isMobile() ){
      var carousel = $('.js-items .js-itemsCarousel');
      
      var headerWidth = $('.js-mobileCollectionInfo').width();
      carousel.find('.js-viewport').width( headerWidth );
      carousel.find('li').width( headerWidth ); 
      
      carousel.tinycarousel();
      
      carousel.bind('move', function(e, currentCard){
        var item = $(currentCard).find('.js-item');
        
        carousel.find('.js-viewport').height( Tisket.Layout.containerFullHeight(item) );
      });
      
      carousel.on('swipeleft', function(event){
        carousel.find('.js-next').click();
      });
      carousel.on('swiperight', function(){
        carousel.find('.js-prev').click();
      });
    }
  };
  
  var extractOptions = function(options){
    messageId = options.messageId;
    currentUserId = options.currentUserId;
  };
  
  var initRecipients = function(){
    initAddNewReceipients();
    
    $('.js-recipientsContainer').find('.js-tooltip').tooltip();
  };
  
  var initNewItemTrigger = function(){
    selectedImage = null;
    
    var product = $('.js-product');
    var itemsCarousel = $('.js-items .js-carousel');
    
    var newItemForm = $('.js-newItemForm');
    var newItemField = newItemForm.find('.js-newItemField'); 
    var newItemTrigger = newItemForm.find('.js-addItemTrigger');
    var newItemCancelTrigger = newItemForm.find('.js-cancelTrigger'); 
    
    newItemForm.find('.js-tooltip').tooltip();
    newItemField.parents('div.js-hover').hover(function(){
      newItemForm.find('.js-tooltip').tooltip('show');  
    }, function(){
      newItemForm.find('.js-tooltip').tooltip('hide');
    });
    
    newItemCancelTrigger.click(function(){
      newItemCancelTrigger.parent().hide();
      
      product.hide();
      itemsCarousel.show();
    });
    
    var addItem = function(){
      var url = newItemField.val();
      
      product.html( $('.js-newProduct').html() );
      var newItemSpinner = product.find('.js-spinner');
      
      newItemSpinner.show();
      product.show();
      
      newItemField.val('');
      
      newItemField.attr('disabled', true);
      
      Tisket.Notifications.hide();
      
      var form = newItemTrigger.parents('form');
      
      var formData = {
        authenticity_token: Tisket.Layout.authenticityToken,
        url: url
      };
      
      newItemField.attr('disabled', true);
      
      setTimeout(function(){
        if( product.is(':hidden') )
          return;
        
        // Hide New Product Item
        newItemSpinner.hide();
        newItemField.val('').attr('disabled', false);
        
        // Show Temporary Card
        var temporalItem = $('<div>').html($('.js-temporalItemTemplate').html()).find('.js-temporalItem').clone();
        temporalItem.insertAfter( product );
        product.hide();
        
        // Store Url mapping to card
        temporalItems[url] = temporalItem;
      }, additionThreshold);
      $.ajax({
        url: (form.attr('action') + '.js'),
        type: form.attr('method'),
        dataType: 'html',
        data: formData,
        success: function(response){
          var addedItem = $('<div>').html(response).find('.js-item');
          var temporalItem = temporalItems[url];
          
          if( temporalItem ){
            addedItem.insertAfter( temporalItem );
            temporalItem.remove();
            delete temporalItems[url];
            
            initItem(addedItem);
          }
          else{
            addedItem.insertAfter( $('.js-product') );
            initItem(addedItem);
            
            // Hide New Product Item
            newItemSpinner.hide();
            newItemField.val('').attr('disabled', false);
            product.hide();
          }
        },
        error: function(jqXHR){
          var response = JSON.parse(jqXHR.responseText);
          
          if(response.item_html){
            var addedItem = $('<div>').html(response.item_html).find('.js-item');
            
            addedItem.insertAfter( $('.js-product') );
            initItem(addedItem);
            
            // Hide New Product Item
            newItemSpinner.hide();
            newItemField.val('').attr('disabled', false);
            product.hide();
          }
          
          Tisket.Notifications.showErrorMessage(response.error_message, 3000);
          
          newItemSpinner.hide();
          newItemField.val('').attr('disabled', false);
          product.hide();
        }
      });
      
      Tisket.Layout.adjustFooterPosition();
    };
    
    var onPaste = function(field){
      setTimeout(function(){
        addItem();
      }, 100);
    };
    
    newItemField.click(function(){
      $(this).val('');
    });
    
    newItemField.keydown(function(e){
      if(e.keyCode == 13){
        e.preventDefault();
        
        addItem();
      }
    });
    
    newItemField.bind('paste', function(){
      onPaste( $(this) );
    });
    
    if( $('.js-itemThumbnail').length == 0 )
      newItemField.focus();
  };
  
  var initNewCommentForm = function(){
    var initForm = function(commentForm, onAddition){
      var commentField = commentForm.find('.js-commentField');
      var itemIdField = commentForm.find('.js-itemId');
      
      commentForm.find('.js-tooltip').tooltip();
      commentField.parents('div.js-hover').hover(function(){
        commentForm.find('.js-tooltip').tooltip('show');  
      }, function(){
        commentForm.find('.js-tooltip').tooltip('hide');
      });
      
      var onFormSubmit = function(){
        commentForm.submit();
      };
      
      commentField.keydown(function(e){
        if(e.keyCode == 13){
          onFormSubmit();
          
          return false;
        }
      });
      
      commentForm.submit(function(){
        var form = $(this);
        
        if( Tisket.Layout.isMobile() ){
          var itemsCarousel = $('.js-itemsCarousel');
          var currentSlideIndex = itemsCarousel.data("plugin_tinycarousel").slideCurrent;
          var currentItem = itemsCarousel.find('.js-item:eq(' + currentSlideIndex + ')');
          
          itemIdField.val( currentItem.attr('data-itemId') );
        };
        
        // FormData is before request because we disable fields before submit so it will be ignored from serialization
        var formData = form.serialize(true);
        
        commentField.attr('disabled', true);
        
        $.ajax({
          url: (form.attr('action') + '.json'),
          type: form.attr('method'),
          data: formData,
          dataType: 'json',
          success: function(response){          
            form.find('.js-commentField').val("");
            
            var commentTemplate = $('<div>').append( $.parseHTML(response.comment_html) ).find('.js-activity');
            
            var comment = commentTemplate.clone();
            initComment(comment);
            
            if(onAddition)
              onAddition(commentTemplate.clone(), response);
          },
          error: function(){
            alert('Something went wrong while adding new comment');
          },
          complete: function(){
            commentField.attr('disabled', false);
          }
        });
        
        return false;
      });
    };
    
    $('.js-items').find('.js-newCommentForm').each(function(){
      var form = $(this);
      var item = form.parents('.js-item'); 
      
      initForm(form, function(comment, response){
        item.find('.js-comments').prepend( comment );
        
        if( Tisket.Layout.isMobile() ){
          $('.js-itemsCarousel .js-viewport').height( Tisket.Layout.containerFullHeight(item) );
        }
        else
          initItemsHeights();
        
        initComment( comment );
        
        item.find('.js-comments').perfectScrollbar('update');
        
        item.find('.js-commentsCount').html( response.item_comments_count );
      });
    });
  };
  
  var initAddNewReceipients = function(){
    $('.js-addNewReceipientsTrigger').click(function(){
      var trigger = $(this);
      
      Tisket.Overlays.CollectionUpdateOverlay.open({
        url: (trigger.attr('href') + '.js'),
        onSuccess: function(response){
          var activitiesContainer = $('.js-activitiesContainer');
          $(response.activities_html).each(function(){
            activitiesContainer.prepend( this );
          });
  
          $('.js-recipientsContainer').html( response.recipients_html );
          $('.js-recipientsCount').html( response.recipients_count_html );
          $('.js-friendsOverlayHTML').html( response.friends_overlay );
          initRecipients();
        
          window.location.reload();
        },
        onError: function(response){
          overlay.find('.js-loadingMask').addClass('hidden');
          alert('Something went wrong while sending invitations');
        }
      });
      
      return false;
    });
  };
  
  var initItem = function(item){
    var onImageEdit = function(trigger){
      var item = trigger.parents('.js-item');
      
      item.find('.js-loader').show();
      $.ajax({
        url: ('/items/' + item.attr('data-itemId') + '/images.json'),
        type: 'get',
        dataType: 'json',
        success: function(response){
          var image = item.find('.js-image');
          var imagesCarousel = item.find('.js-imagesCarousel');
          var images = imagesCarousel.find('.js-images');
          var imageTemplate = images.find('li:first').clone();
          
          images.find('li').remove();
          $(response.images_urls).each(function(){
            var element = imageTemplate.clone();
            element.find('img').attr('src', this);
            images.append(element);
          });
          image.hide();
          imagesCarousel.show();
          imagesCarousel.tinycarousel();
          
          trigger.html( trigger.attr('data-updateText') );
        },
        error: function(jqXHR){
          alert('Something went wrong!');
        },
        complete: function(){
          item.find('.js-loader').hide();
        }
      });
    };
    
    var onImageUpdate = function(trigger){
      var item = trigger.parents('.js-item');
      var imagesCarousel = item.find('.js-imagesCarousel');
      var images = imagesCarousel.find('.js-images');
      
      item.find('.js-loader').show();
      
      var imageUrl = images.find(
        ('img:eq(' + (imagesCarousel.data('plugin_tinycarousel').slideCurrent) + ')')
      ).attr('src');
      
      $.ajax({
        url: ('/items/' + item.attr('data-itemId') + '/image.json'),
        type: 'post',
        data: {
          authenticity_token: Tisket.Layout.authenticityToken,
          image_url: imageUrl 
        },
        dataType: 'json',
        success: function(response){
          var image = item.find('.js-image');
          
          image.attr('src', response.image_url);
          
          imagesCarousel.hide();
          image.show();
          
          trigger.html( trigger.attr('data-editText') );
        },
        error: function(jqXHR){
          alert('Something went wrong while updating image!');
        },
        complete: function(){
          item.find('.js-loader').hide();
        }
      });
    };
    
    var canEdit = item.attr('data-canEdit');
    
    item.find('.js-url').click(function(){
      var itemId = item.attr('data-itemId');
      
      $.ajax({
        url: '/items/' + itemId + '/clicks.js',
        type: 'post',
        data: {
          authenticity_token: Tisket.Layout.authenticityToken
        }
      });
      
      return true;
    });
    
    item.find('.js-saveTrigger').click(function(){
      var trigger = $(this);
      var item = trigger.parents('.js-item');
      var overlay = $('<div>').html( $('.js-saveItemOverlayTemplate').html() ).find('.js-saveItemOverlay');
      
      overlay.find('.js-image').attr('src', item.find('.js-image').attr('src'));
      overlay.find('form').attr(
        'action', overlay.find('form').attr('action').replace(':item_id', item.attr('data-itemId'))
      );
      
      overlay.on('shown', function(){
        var newCollectionName = overlay.find('.js-newCollectionName');
        var existingCollectionRadio = overlay.find('.js-existingCollectionRadio');
        var newCollectionRadio = overlay.find('.js-newCollectionRadio');
        var form = overlay.find('form');
        
        overlay.find('input:radio').iCheck({
          checkboxClass: 'icheckbox_square-grey',
          radioClass: 'iradio_square-grey',
        });
         
        newCollectionName.focus(function(){
          existingCollectionRadio.iCheck('uncheck');
          newCollectionRadio.iCheck('check');
        });
        
        existingCollectionRadio.on('ifChecked', function(){
          newCollectionRadio.iCheck('uncheck');
          newCollectionName.blur();  
        });
        
        newCollectionRadio.on('ifChecked', function(){
          existingCollectionRadio.iCheck('uncheck');
          newCollectionName.focus();
        });
        
        overlay.find('.js-saveTrigger').click(function(){
          if( existingCollectionRadio.is(':checked') ){
            newCollectionName.attr('disabled', true);
          }
          
          $.ajax({
            url: (form.attr('action') + '.js'),
            type: 'post',
            data: form.serialize(true),
            success: function(){
              overlay.modal('hide');
            },
            error: function(){
              alert('Something went wrong while saving item!');
            }
          });
        });
      });
      
      overlay.modal();
    });
    
    item.find('.js-deleteTrigger').click(function(){
      var trigger = $(this);
      var itemId = item.attr('data-itemId');
      
      var overlay = $(
        '<div class="modal hide fade confirmationMsg" tabindex="-1" data-color="white"></div>'
      );
      overlay.html( item.find('.js-removeItemOverlayHTML').html() );
      overlay.on('shown', function(){
        overlay.find('.js-removeTrigger').click(function(){
          $.ajax({
            url: ('/collections/' + messageId + '/delete_item/' + itemId + '.js'),
            type: 'delete',
            data: {authenticity_token: Tisket.Layout.authenticityToken},
            success: function(){
              window.location.reload();
            },
            error: function(){
              Tisket.Notifications.showErrorMessage('Something went wrong while deleting the item');
            }
          });
        });
      });
      overlay.modal({
        keyboard: false
      });
      
      return false;
    });
    
    item.find('.js-editImageTrigger').click(function(){
      var trigger = $(this);
      
      if( trigger.html() == trigger.attr('data-editText') ){
        onImageEdit( trigger );
      }
      else{
        onImageUpdate( trigger );
      }
    });
    
    if( canEdit == "true" ){
      $(['brand', 'title', 'price', 'description']).each(function(){
        var fieldName = this;
        
        var container = item.find('.js-' + this); 
        var field = item.find('.js-' + this + 'Field');
        var trigger = item.find('.js-' + this + 'EditTrigger');
        
        var onClick = function(){
          container.hide();
          
          //field.val( container.html().replace(/<a.*?>(.*?)<\/a>/g, '$1').br2nl() );
          var displayedValue = container.html().br2nl();
          if( displayedValue != field.attr('placeholder') ) 
            field.val( displayedValue );
          field.show();
          field.focus();
        };
        
        container.click( onClick );
        trigger.click( onClick );
        
        var onEnter = function(){
          var data = {
            authenticity_token: Tisket.Layout.authenticityToken
          };
          data[('item[' + fieldName + ']')] = field.val();
        
          $.ajax({
            url: ('/items/' + item.attr('data-itemId') + '.js'),
            type: 'put',
            dataType: 'json',
            data: data,
            success: function(){
              field.hide();
              
              if( field.val().length == 0 )
                container.html( field.attr('placeholder') );
              else{
                container.html( field.val().nl2br() );
              }
              
              container.show();
            },
            error: function(){
              Tisket.Notifications.showErrorMessage('Something went wrong while updating item!');
            }
          });
        };
        
        field.keydown(function(event){
          if(event.keyCode == 13){
            event.preventDefault();
            
            onEnter();
          }
        });
        field.blur( onEnter );
      });
    }
       
    initItemLikeActions(item);
    
    item.find('.js-comments .js-activity').each(function(){
      var comment = $(this);
      initComment( comment );
    });
    
    if (!(Tisket.Layout.isMobile() || Tisket.Layout.isTablet())){
      item.draggable({ 
        opacity: 1,
        helper: "clone",
        cursor: "default",
        cursorAt: {top: 15, left: 7},
        helper: function(){
          return $("<div class='smallPlaceholder'></div>");
        },
        start: function(){
          var draggedItem = $(this);
          
          $(".js-addProduct, .js-newCollectionTrigger, header").animate({
            opacity: 0.3,
          });
          
          $(".js-item").each(function(){
            var item = $(this);
            
            if( item.attr('data-itemId') != draggedItem.attr('data-itemId') ){
              item.animate({
                opacity: 0.3,
              }); 
            }
          });
        },
        stop: function(){
          $(".js-addProduct, .js-newCollectionTrigger, header").animate({
            opacity: 1,
          });
          $(".js-item").animate({
            opacity: 1,
          });
        }
      });
    }
    
    if( Tisket.Layout.isMobile() )
      item.find('.js-productImage').unwrap();
  };
  
  var initItemLikeActions = function(item){
    initItemLikeTriggers( item );
  };
  
  var initItemLikeTriggers = function(item){
    item.find('.js-likeActions').find('.js-likeTrigger').click(function(){
      var trigger = $(this);
      
      if( trigger.attr('data-disabled') )
        return false;
      
      trigger.attr('data-disabled', 'true');
      
      $.ajax({
        url: (trigger.attr('href') + '.json'),
        type: 'put',
        data: { authenticity_token: Tisket.Layout.authenticityToken },
        dataType: 'json',
        success: function(response){
          item.find('.js-likeActions').html( response.item_like_actions_html );
          initItemLikeActions( item );
        },
        error: function(){
          trigger.removeAttr('data-disabled');
          
          Tisket.Notifications.showErrorMessage('Something went wrong while liking item');
        }
      });
      
      return false;
    });
    
    item.find('.js-likeActions').find('.js-unlikeTrigger').click(function(){
      var trigger = $(this);
      
      if( trigger.attr('data-disabled') )
        return false;
      
      trigger.attr('data-disabled', 'true');
      
      $.ajax({
        url: (trigger.attr('href') + '.json'),
        type: 'put',
        data: {authenticity_token: Tisket.Layout.authenticityToken},
        dataType: 'json',
        success: function(response){
          item.find('.js-likeActions').html( response.item_like_actions_html );
          initItemLikeActions( item );
        },
        error: function(){
          trigger.removeAttr('data-disabled');
          
          alert('Something went wrong while unliking the item');
        }
      });
      
      return false;
    });
    
    item.find('.js-likeActions').find('.js-leaveTrigger').click(function(){
      var trigger = $(this);
      
      if( trigger.attr('data-disabled') )
        return false;
      
      trigger.attr('data-disabled', 'true');
      
      $.ajax({
        url: (trigger.attr('href') + '.json'),
        type: 'put',
        data: { authenticity_token: Tisket.Layout.authenticityToken },
        dataType: 'json',
        success: function(response){
          item.find('.js-likeActions').html( response.item_like_actions_html );
          initItemLikeActions( item );
        },
        error: function(){
          trigger.removeAttr('data-disabled');
          
          Tisket.Notifications.showErrorMessage('Something went wrong while leaving item');
        }
      });
      
      return false;
    });
    
    item.find('.js-likeActions').find('.js-unleaveTrigger').click(function(){
      var trigger = $(this);
      
      if( trigger.attr('data-disabled') )
        return false;
      
      trigger.attr('data-disabled', 'true');
      
      $.ajax({
        url: (trigger.attr('href') + '.json'),
        type: 'put',
        data: {authenticity_token: Tisket.Layout.authenticityToken},
        dataType: 'json',
        success: function(response){
          item.find('.js-likeActions').html( response.item_like_actions_html );
          initItemLikeActions( item );
        },
        error: function(){
          trigger.removeAttr('data-disabled');
          
          alert('Something went wrong while unleaving the item');
        }
      });
      
      return false;
    });
  };
  
  var initComment = function(comment){
    var getItemComment = function(comment){
      var activityId = comment.attr('data-activityId');

      if( itemId ){
        var item = $('.js-items .js-item[data-itemId=' + itemId + ']');
        var itemComments = item.find('.js-comments');
        if( itemComments.attr('data-loaded') != '0' ){
          var itemComment = item.find('.js-comments .js-activity[data-activityId=' + activityId + ']');
          return itemComment; 
        }
      }
      
      return null;
    };
    
    var updateComment = function(response){
      var commentBody = $('<div>').append( $.parseHTML(response.activities_html) ).find('.js-activity').html();
      
      comment.html( commentBody );
      initComment( comment );
    };
    
    var loadLikes = function(comment){
      $.ajax({
        url: ('/activities/' + comment.attr('data-activityId') + '/likes.js'),
        type: 'get',
        dataType: 'json',
        success: function(response){
          comment.find('.js-likesCount').attr('data-original-title', response.names.join(', '));
          comment.find('.js-likesCount').tooltip('show');
        },
        error: function(){
          alert('Something went wrong while unliking comment');
        }
      });
    };
    
    comment.find('.js-deleteTrigger').click(function(){
      var trigger = $(this);
      
      $.ajax({
        url: (trigger.attr('href') + '.json'),
        type: 'delete',
        data: {authenticity_token: Tisket.Layout.authenticityToken},
        dataType: 'json',
        success: function(){
          comment.remove();
        },
        error: function(){
          alert('Something went wrong while deleting comment');
        }
      });
      
      return false;
    });
    
    comment.find('.js-likeTrigger').click(function(){
      $.ajax({
        url: ('/activities/' + comment.attr('data-activityId') + '/like.js'),
        type: 'post',
        data: {
          'authenticity_token': Tisket.Layout.authenticityToken
        },
        dataType: 'json',
        success: function(response){
          updateComment(response);
        },
        error: function(){
          alert('Something went wrong while liking comment');
        }
      });
      
      return false;
    });
    comment.find('.js-unlikeTrigger').click(function(){
      $.ajax({
        url: ('/activities/' + comment.attr('data-activityId') + '/unlike.js'),
        type: 'post',
        data: {
          'authenticity_token': Tisket.Layout.authenticityToken
        },
        dataType: 'json',
        success: function(response){
          updateComment(response);
        },
        error: function(){
          alert('Something went wrong while unliking comment');
        }
      });
      
      return false;
    });
    
    comment.find('.js-likesCount').tooltip();
    
    comment.find('.js-likesCount').hover(function(){
      var trigger = $(this);
      
      if( trigger.attr('data-original-title').length == 0 ){        
        loadLikes(comment); 
      }
    });
    
    comment.find('.js-content').html( textWithLinks(comment.find('.js-content').html()) );
    
    comment.find('.js-timeago').timeago();
  };
  
  var textWithLinks = function(text){
    if( text == undefined )
      return text;
    
    var url_regex = new RegExp(
      /(((http)(s?):\/\/(www\.)?)?([-a-zA-Z0-9@:%_\+.~#?&//=]{2,256}\.[a-z]{2,4}\b(\/[-a-zA-Z0-9@:%_\+.~#?&//=]*)?))/gi
    );
    
    text = text.replace(url_regex, function(match, p1, p2, p3, p4, p5, p6, p7, offset, s){
      if( match.match(/\.\.+/) )
        return match;
      else
        return ['<a href="http', p4, '://', p5, p6, '" target="_blank">', p1, '</a>'].join('');
    });
    
    return text;
  };
  that.textWithLinks = textWithLinks;
  
  return that;
}());