class ItemsController < ApplicationController
  before_filter :check_bookmarklet, only: [:scraping, :add_to_existing_messages, :add_to_new_message,
                                           :post_to_existing_messages, :post_to_new_message]

  before_filter :update_item_authorization, :only => :update

  # GET /items/1
  # GET /items/1.json
  def show
    @item = Item.find(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @item }
    end
  end  

  def update
    # prepare @item from "update_item_authorization" filter
    
    @item.attributes = params[:item]
    
    begin
      @item.save!
      
      respond_to do |format|
        format.js { render :json => {}, :status => 200 }
      end
    rescue Exception
      respond_to do |format|
        format.js { render :json => {}, :status => 500 }
      end
    end
  end

  # GET /items/scrap.js
  # GET /items/scrap.html
  # displays a form for the user to input the URL to scrap.
  # Step 1
  def scrap
    if params[:new_item] == '1'
      current_user.draft_item.try(:destroy)
      current_user.draft_item = nil
    end
    
    if params[:bookmarklet] == 'true'
      @url = params[:url]
    else
      @url = current_user.draft_item.try(:data).try(:url) || ''
    end
    
    # params[:message_id] is set if user clicked add more items in message's view
    @message_id ||= params[:message_id]
    
    respond_to do |format|
      format.html do
        if params[:bookmarklet] == 'true'
          render 'step_bookmarklet_loading', :layout => 'bookmarklet'
        else
          render 'step_add_url'
        end 
      end
      
      format.js { render :partial => 'step_add_url', :status => 200 }
    end
  end
  
  def scraping_run
    data = scraping_result
    
    respond_to do |format|
      format.js { render :nothing => true, :status => 200 }
    end
  end
  
  # GET /items/scraping.js
  # GET /items/scraping.html
  # we can reach here from one way of the following:

  # (1) after Step 1 to scrap the url and extract item's information,
  # in this case the scraped data (extracted information) is stored in the draft item.
  
  # (2) from the bookmarklet, when the user click it from his browser, then we
  # scrap the url of the page and extract item's information,
  # in this case the scraped data (extracted information) is stored in the draft item.
  # also params[:bookmarklet] = 1

  # (3) we can went back to here from Step 3 to edit some attributes,
  # in this case the data is retrieved from the draft item.
  # Step 2
  def scraping
    data = scraping_result

    if data.present?
      # retreiving from the draft Item
      @item = current_user.items.new(data.item) if data.item
      @prefered_image_url = (params[:biggest_image_url] || data.prefered_image_url)
      # selected_image_url existis only if coming Back from Step 3
      # and it is the image the user chose for the product in Step 2
      @selected_image_url = data.selected_image_url
      @images_urls = data.images_urls || []
    end
    
    # if @Item is nil, then the scraping failed, or the draft item was deleted (shouldn't occured)
    # in this case, go back to step 1 and repeat the process
    # otherwise continue in Step 2 to edit attributes and choose item's image

    # params[:message_id] is set if user clicked add more items in message's view
    # i.e. when a message's user recommends an item
    @message_id = params[:message_id]
    
    respond_to do |format|
      if @item
        format.html do
          render 'step_item_info', :layout => (@bookmarklet ? 'bookmarklet' : 'application'), :status => 200
        end
        format.js { render :partial => 'step_item_info', :status => 200 }
      else
        if @bookmarklet
          format.html do
            render 'step_bookmarklet_non_product_page', :layout => (@bookmarklet ? 'bookmarklet' : 'application'),
                                                        :status => 500
          end
        else
          @error = I18n.t('item.scraping_field')
          
          format.js { render :partial => 'step_add_url', :status => 500 }
        end
      end
    end
  end

  # GET /items/add_to_existing_messages.js
  # GET /items/add_to_existing_messages.html
  # from Step 2 we can reach here to add the item to already exists messages
  # Step 3.a
  def add_to_existing_messages
    @selected_image_url = params[:item].delete(:image)
    @item = current_user.items.new(params[:item])
    
    draft = DraftItem.find_or_initialize_by_user_id(current_user.id)
    data = draft.data

    respond_to do |format|
      if @item.valid?
        # save item's current state, it is useful if the user went back to Step 2
        data.item = params[:item]
        data.selected_image_url = @selected_image_url
        draft.save
        # messages that the user own or involved in and can hold the item (active)
        @owned_messages = current_user.owned_messages.active
        @involved_messages = current_user.involved_messages.active
        
        format.js{ render :partial => 'step_add_to_existing_message', :status => 200 }
      else
        # the item is not in a valid state so go back to Step 2 
        # and display error messages to the user to correct.
        @prefered_image_url = data.prefered_image_url
        @selected_image_url = data.selected_image_url
        @images_urls = data.images_urls || []
        
        format.js{ render :partial => 'step_item_info', :status => 500 }
      end
    end
  end

  # GET /items/add_to_new_message.js
  # GET /items/add_to_new_message.html
  # from Step 2 we can reach here to add the item to a new message
  # Step 3.b
  def add_to_new_message
    @selected_image_url = params[:item].delete(:image)
    @item = current_user.items.new(params[:item])

    draft = DraftItem.find_or_initialize_by_user_id(current_user.id)
    data = draft.data

    respond_to do |format|
      if @item.valid?
        # save item's current state, it is useful if the user went back to Step 2
        data.item = params[:item]
        data.selected_image_url = @selected_image_url
        draft.save
        # build new message so that the item can be added to
        @item.build_message
        
        format.js{ render :partial => 'step_add_to_new_message', :status => 200 }
      else
        # the item is not in a valid state so go back to Step 2 
        # and display error messages to the user to correct.
        @prefered_image_url = data.prefered_image_url
        @selected_image_url = data.selected_image_url
        @images_urls = data.images_urls || []
        
        format.js{ render :partial => 'step_item_info', :status => 500 }
      end
    end
  end

  # POST /items/post_to_existing_messages.js
  # POST /items/post_to_existing_messages.html
  # create the item and add it to selected messages.
  def post_to_existing_messages
    # before appending the item to a message make sure that it is active and blongs to him
    msg_id = params[:item][:message_id]
    current_user.total_messages.active.find msg_id if msg_id.present?
    
    @item = current_user.items.new(params[:item].except(:image))
    @item.image = URI.parse(params[:item][:image]) if params[:item][:image].present?
    respond_to do |format|
      if @item.save
        current_user.draft_item.destroy
        current_user.activities.create(item_id: @item.id, message_id: @item.message.id,
                                       kind: Activity::COMMENT, activity: params[:comment])
                                       
        format.js do
          if @bookmarklet
            render :json => {bookmarklet: true, html: render_to_string(:partial => 'step_bookmarklet_success')},
                   :status => 200
          else
            render :json => {bookmarklet: false, location: message_path(@item.message)},
                   :status => 200
          end
        end
      else
        format.js do
          @selected_image_url = params[:item][:image]
      
          draft = DraftItem.find_or_initialize_by_user_id(current_user.id)
          data = draft.data
          
          @owned_messages = current_user.owned_messages.active
          @involved_messages = current_user.involved_messages.active
          
          render :partial => 'step_add_to_existing_message', :status => 500
        end
      end
    end
  end

  # POST /items/post_to_new_message.js
  # POST /items/post_to_new_message.html
  # create the item and add it to newly created message.
  def post_to_new_message
    require_dependency 'emails_tokenizer'
    params[:item][:recipients] = EmailsTokenizer.tokenize( (params[:recipients] || []).join(',') )
    
    @item = current_user.items.new(params[:item].except(:image))
    @item.message.owner = current_user
    @item.image = URI.parse(params[:item][:image]) if params[:item][:image].present?
    @item.add_to_new_message = true
    
    respond_to do |format|
      if @item.save
        current_user.draft_item.destroy if current_user.draft_item
        # current_user is the owner of the newly created message
        current_user.owned_messages << @item.message
        
        format.js do
          if @bookmarklet
            render :json => {bookmarklet: true, html: render_to_string(:partial => 'step_bookmarklet_success')},
                   :status => 200
          else
            render :json => {bookmarklet: false, location: message_path(@item.message)},
                   :status => 200
          end
        end
      else
        # something went wrong, so go back to Step 2
        format.js do
          @selected_image_url = params[:item].delete(:image)
      
          draft = DraftItem.find_or_initialize_by_user_id(current_user.id)
          data = draft.data
          
          render :partial => 'step_add_to_new_message', :status => 500
        end
      end
    end
  end
  
  protected
  
  def scraping_result
    draft = DraftItem.find_or_initialize_by_user_id(current_user.id)
    data = draft.data # data is OpenStruct
    
    if params[:url] and params[:url] != data.url # new item (come From Step 1 or bookmarklet)
      if scraped_data = Scraper.scrap(params[:url])
        # Storing the extracted data in draft Item
        data.url = params[:url]
        data.item = scraped_data.except(:prefered_image_url, :images_urls)
        data.prefered_image_url = scraped_data[:prefered_image_url]
        data.images_urls = scraped_data[:images_urls]
        data.selected_image_url = nil
        draft.save
      else
        # destroy the draft Item if invalid URL  
        draft.destroy
        data = nil
      end
    end
    
    data
  end
  
  def check_bookmarklet
    # bookmarklet is defined if the user came here through the browser's bookmarklet
    @bookmarklet = params[:bookmarklet] == 'true'
  end
  
  def update_item_authorization
    @item = Item.find( params[:id] )
    
    return true if current_user.can_edit?(@item)
    
    raise Exception.new("Unauthorized!")
  end
end
