class Messages::ItemsController < ApplicationController
  before_filter :authorize_user!
  
  def create
    # @message is loaded from "authorize_user!" filter
    
    begin
      # Create Item with just URL
      @item = @message.items.build(url: params[:url])
      @item.user = current_user
      @item.processed = false
      @item.save!
      
      begin
        # Scrap Data and Add to Item & save!
        info = Scraper.scrap( @item.url )
        if info
          ['title', 'price', 'brand', 'description'].each do |field_name|
            @item.send("#{ field_name }=", info[field_name.to_sym])
          end
          @item.image_url = (info[:prefered_image_url] || info[:images_urls].first)
          @item.processed = true
          @item.save!
        end
        
        respond_to do |format|
          format.js { render partial: 'messages/items', locals: {items: [@item]}, :status => 200 }
        end
      rescue Exception => e
        respond_to do |format|
          format.js do
            render json: {
              item_html: render_to_string(partial: 'messages/items', locals: {items: [@item]}),
              error_message: 'Item was added but a problem occurred while grabbing its details!'
            }, status: 500
          end
        end
      end
    rescue ActiveRecord::RecordInvalid => e
      Rails.logger.debug ">>>>>>>>>>>>>>>>>>> #{ e.message }"
      
      if e.record.errors[:url].empty?
        begin
          FailedUrl.create!({
            url: e.record.url, user_id: current_user.try(:id),
            failure_reason_id: FailedUrl::FAILURE_REASONS[:crawl_failed].id
          })
        rescue => e
          Airbrake.notify_or_ignore(e)
        end
        
        error_message = "A problem happened while retrieving item details and was reported to support team!"
      else
        begin
          FailedUrl.create!({
            url: e.record.url, user_id: current_user.try(:id),
            failure_reason_id: FailedUrl::FAILURE_REASONS[:invalid_url].id
          })
        rescue => e
          Airbrake.notify_or_ignore(e)
        end
        
        error_message = "This doesn't look like a valid URL, please check address and try again."
      end
      
      respond_to do |format|
        format.js do
          render :json => {
            error_message: error_message
          }, :status => 406
        end
      end
    end
  end
  
  #-------#
  protected
  #-------#
  
  def authorize_user!
    @message = Message.find( params[:message_id] )
    
    return true if current_user.can_post_item?( @message )
    
    respond_to do |format|
      format.js { render :json => {}, :status => 403 }
    end
  end
end