class Users::InvitationsController < Devise::InvitationsController
  before_filter :redirect_to_open_messages_if_logged_in, :only => :new 
	
	def new
    build_resource
    
    respond_to do |format|
      format.html
    end
  end
	
	# POST /resource/invitation
  def create
    self.resource = resource_class.invite!(resource_params, current_inviter)  do |r|
		  r.skip_invitation = true
		end

    if resource.errors.empty?
      @message = "We'll send you an email to register soon."
      
      respond_to do |format|
        format.js
        
        format.json do
          render :json => {
            :message => @message
          }, :status => 200
        end
      end
    else
      if resource.errors[:email].include?('has already been taken')
        @message = "You're already on the list."
      else
        @message = "Please enter valid email address."
      end
      
      respond_to do |format|
        format.js
        
        format.json do
          render :json => {
            :message => @message
          }, :status => 406
        end
      end
    end
  end

  # GET /resource/invitation/accept?invitation_token=abcdef&msg=2
  def edit
    @user = User.find_by_invitation_token params[:invitation_token]
    @message_id = params[:msg]
    
    render :edit
  end

  # PUT /resource/invitation
  def update
    self.resource = resource_class.accept_invitation!(resource_params)

    if resource.errors.empty?
      sign_in(resource_name, resource)
      # if params[msg_id] exists then the user was invited to a message
      # then set the friendship and redirect to show this message
      # else go to after_accept_path_for(resource)
      if msg = resource.involved_messages.find_by_id(params[:msg_id])
        redirect_to accept_invitation_message_path(msg)
      else
        flash_message = resource.active_for_authentication? ? :updated : :updated_not_active                                                                                        
        set_flash_message :notice, flash_message
        respond_with resource, :location => after_accept_path_for(resource)
      end
    else
      respond_with_navigational(resource){ render :edit }
    end
  end
  
  #-------#
  protected
  #-------#
  
  def redirect_to_open_messages_if_logged_in
    return unless user_signed_in?
    
    respond_to do |format|
      format.html { redirect_to open_messages_path }
    end
  end
  
  def after_accept_path_for(resource)
    open_messages_path(:new_user => 1)
  end
end