class Users::OmniauthCallbacksController < Devise::OmniauthCallbacksController
  def google
    authenticate
  end

  def facebook
    create
  end

  def create
    auth = request.env["omniauth.auth"]

    unless request.env["omniauth.params"]["format"].blank?
      request.format = request.env["omniauth.params"]["format"]
    end

    if request.env["omniauth.params"]["invitation_token"]
      invited_user = User.find_by_invitation_token( request.env["omniauth.params"]["invitation_token"] )
    else
      invited_user = nil
    end
    message_id = request.env["omniauth.params"]["msg"]

    authentication = UserAuthentication.find_by_provider_and_uid(auth.provider, auth.uid)
   
    if authentication
      flash[:notice] = "Signed in successfully."
      
      if message_id
        MessageUserRelation.find_or_create_by_message_id_and_user_id(message_id, authentication.user_id)
        if invited_user
          MessageUserRelation.find_by_message_id_and_user_id(message_id, invited_user.id).destroy
        end
        
        sign_in(:user, authentication.user)
        
        respond_to do |format|
          format.html { redirect_to accept_invitation_message_path(message_id) }
          format.extchr { chrome_extension_response }
        end
      else
        respond_to do |format|
          format.html { sign_in_and_redirect(:user, authentication.user) }
          format.extchr do
            sign_in(:user, authentication.user)
            chrome_extension_response
          end
        end
      end
    else
      # Authentication not found, thus a new user.
      
      Rails.logger.debug ">>>>>>>>>>>>>>>>>> #{ request.env["omniauth.params"] }"
      
      # Find user by email then if invited by invitation token then create new with this email address
      user = (User.find_by_email(auth.info.email) || invited_user || User.new(:email => auth.info.email))
      
      user.apply_omniauth(auth)
      
      if user.new_record? && request.env["omniauth.params"]["referral"]
        referral = User.find_by_id( request.env["omniauth.params"]["referral"] )
        if referral
          user.invited_by_id = referral.id
          user.invitation_medium = request.env["omniauth.params"]["source"]
        end
      end
      
      if user.save(:validate => false)
        if user.invited_to_sign_up?
          user.accept_invitation!  
        end
        
        user.store_facebook_friends!
        
        flash[:notice] = "Account created and signed in successfully."
        
        if message_id
          MessageUserRelation.find_or_create_by_message_id_and_user_id(message_id, user.id)
          if invited_user && user.id != invited_user.id
            MessageUserRelation.find_by_message_id_and_user_id(message_id, invited_user.id).destroy
          end
          
          sign_in(:user, user)
          
          respond_to do |format|
            format.html { redirect_to accept_invitation_message_path(request.env["omniauth.params"]["msg"]) }
            format.extchr { chrome_extension_response }
          end
        else
          respond_to do |format|
            format.html { sign_in_and_redirect(:user, user) }
            format.extchr { chrome_extension_response }
          end
        end
      else
        respond_to do |format|
          format.html do
            flash[:error] = "Error while creating a user account. Please try again."
            redirect_to root_url 
          end
          format.extchr { chrome_extension_response }
        end
      end
    end
  end
  
  #-------#
  protected
  #-------#
  
  def chrome_extension_response
    render action: 'success', content_type: 'text/html', status: 200
  end
  
  def authenticate
    auth = request.env["omniauth.auth"]
    
    response_type = env["omniauth.params"]['response_type']
    
    if current_user
      authentication = current_user.authentications.google.first
      user = current_user
    else
      authentication = UserAuthentication.find_by_provider_and_uid(auth.provider, auth.uid)
    end
    
    if authentication
      if response_type == 'js'
        render :template => 'shared/successful_authentication', :status => 200
      else
        sign_in_and_redirect(:user, authentication.user)
      end
    else
      # Authentication not found, thus a new user, or using another provider
      user ||= User.find_or_initialize_by_email(auth.info.email)
      
      invitation_token = env["omniauth.params"]['invitation_token']
      
      if user.new_record? || (user.invited_to_sign_up? && invitation_token.nil?)
        if response_type == 'js'
          render :template => 'shared/authentication_failed', :status => 200
        else
          if user.new_record?
            flash[:error] = "You need to request invitation to get access!"
          else
            flash[:error] = "You need to wait until you are granted access!"
          end
          
          redirect_to :back
        end
      else
        user.apply_omniauth(auth)
        
        begin
          user.transaction do
            user.save!
            
            unless invitation_token.blank?
              user.reload
              user.accept_invitation!  
            end
          end
          
          if response_type == 'js'
            render :template => 'shared/successful_authentication', :status => 200
          else
            sign_in_and_redirect(:user, user)
          end
        rescue ActiveRecord::RecordInvalid => e
          Rails.logger.debug ">>>>>>>>>>>>> #{ user.errors.full_messages }"
          
          if response_type == 'js'
            render :template => 'shared/authentication_failed', :status => 200
          else
            flash[:error] = "Something went wrong while authenticating!!"
            redirect_to :back
          end
        rescue Exception => e
          Airbrake.notify_or_ignore(e)
          
          if response_type == 'js'
            render :template => 'shared/authentication_failed', :status => 200
          else
            flash[:error] = "Something went wrong while authenticating!!"
            redirect_to :back
          end
        end
      end
    end
  end
end