class Mailer < ActionMailer::Base
  default from: "Tisket Team <contact@tisket.com>"

  # Subject can be set in your I18n file at config/locales/en.yml
  # with the following lookup:
  #
  #   en.mailer.send.subject
  #

  #-New Emails-----
    def invitation(inviter, recipients, collection, new_user = false)
      return if recipients.empty?
      
      @inviter = inviter
      @collection = collection
      
      @new_user = new_user
      @user = recipients.first if @new_user
      
      mail(to: recipients.collect(&:email), subject: "#{ @inviter.name } invited you to a collection on Tisket!") do |format|
        format.html { render :layout => 'new_mailer' }
      end
    end
    
    def new_item(poster, recipients, collection, item, new_user = false)
      return if recipients.empty?
      
      @poster = poster
      @collection = collection
      @item = item
      
      @new_user = new_user
      @user = recipients.first if @new_user
      
      mail(to: recipients.collect(&:email), subject: "New Activity on Tisket") do |format|
        format.html { render :layout => 'new_mailer' }
      end
    end
    
    def new_comment(commenter, recipients, collection, comment, new_user = false)
      return if recipients.empty?
      
      @commenter = commenter
      @collection = collection
      @comment = comment
      
      @new_user = new_user
      @user = recipients.first if @new_user
      
      mail(to: recipients.collect(&:email), subject: "New Activity on Tisket") do |format|
        format.html { render :layout => 'new_mailer' }
      end
    end
    
    def new_item_like(item, user)
      @item = item
      @user = user
      
      mail(to: [@item.user.email], subject: "#{ @user.name } likes an item you posted on Tisket!") do |format|
        format.html { render :layout => 'new_mailer' }
      end
    end
  #-----

  def new_item_added_to_existing_messages(message, item)
    @message = message
    @item = item
    @activity = @item.activities.find_by_message_id(@message.id).try(:activity)
    @owner = item.user
    
    @purpose = 'new_item_added_to_existing_messages'

    recipients = @message.total_users.inject([]) do |addresses, u|
      # send a notification email to message's members except the item creator
      # and members who chosed not to receive such notifications 
      if(u != @owner && (u.new_item_notifications? || u.all_notifications?))
        addresses << u.email
      end
      addresses
    end

    if recipients.length > 0
      mail(to: recipients, subject: "#{ @owner.name } needs your opinion!") do |format|
        format.html { render 'new_item', :layout => 'new_mailer' }
      end
    end
  end

  def new_item_at_new_message_existing_users(message, item, inviter, recipients)
    @message = message
    @item = item || @message.items.first || Item.new
    
    @purpose = 'new_item_at_new_message_existing_users'
    
    mail(to: recipients, subject: "#{ inviter.name } needs your opinion!") do |format|
      format.html { render 'new_item', :layout => 'new_mailer' }
    end
  end

  def new_item_at_new_message_new_users(message, item, inviter, recipient)
    @message = message
    @item = item || @message.items.first || Item.new
    @user = recipient
    
    @purpose = 'new_item_at_new_message_new_users'
    
    mail(to: @user.email, subject: "#{ inviter.name } needs your opinion!") do |format|
      format.html { render 'new_item', :layout => 'new_mailer' }
    end
  end

  def daily_digest_mail(user)
    if user.all_notifications? && !user.invited?
      daily_summary = user.daily_summary
      
      @user = user
      @digest = daily_summary[:digest]
      @purchased_items = daily_summary[:purchased_items]
      
      if @digest.present? || @purchased_items.present?
        mail to: @user.email, subject: "New Activity on Tisket!"
      end
    end
  end
end