class Activity < ActiveRecord::Base
  #-Constants-----
    LIKE = 'like'
    LEAVE = 'leave'
    COMMENT = 'comment'
    CREATE_ITEM = 'create_item'
    ACCEPT_INVITATION = 'accept_invitation'
    SEND_INVITATION = 'send_invitation'
  #-----

  #-Attributes-----
    attr_accessible :activity, :item_id, :message_id, :kind, :performed_by, :visible
  #-----

  #-Associations-----
    belongs_to :message
    belongs_to :item
    belongs_to :user
    
    belongs_to :performed_by, :class_name => 'User'
    
    has_many :likes, :class_name => 'ActivityLike', :dependent => :destroy
  #-----
  
  #-Validations-----
    validates :activity, presence: true, if: :comment?
    validates :message, presence: true
  #-----

  #-Callbacks-----
    after_create :update_message_last_activity_time, :increment_users_activities_count
    
    after_create do |activity|
      activity.item.increment!(:comments_count) if activity.comment? && activity.item_id.present?
    end
    
    # after_create do |activity|
      # return true if activity.message.public?
      
      # if activity.comment?
        # recipients = (activity.message.total_users - [activity.user])
        # recipients = recipients.select {|recipient|
          # recipient.all_notifications?
        # }
#         
        # new_users = []
        # existing_users = []
        # recipients.each do |recipient|
          # if recipient.invited_to_sign_up?
            # new_users << recipient
          # else
            # existing_users << recipient
          # end
        # end
        
        # if configatron.emails.async
          # Mailer.delay.new_comment(
            # activity.user, existing_users, activity.message, activity.activity
          # )
        # else
          # Mailer.new_comment(
            # activity.user, existing_users, activity.message, activity.activity
          # ).deliver
        # end
        
        # new_users.each do |recipient|
          # if configatron.emails.async
            # Mailer.delay.new_comment(
              # activity.user, [recipient], activity.message, activity.activity, true
            # )
          # else
            # Mailer.new_comment(
              # activity.user, [recipient], activity.message, activity.activity, true
            # ).deliver
          # end
        # end
      # elsif activity.like?
        # if (activity.performed_by_id != activity.item.user_id) && activity.message.has_user?( activity.item.user_id )
          # if configatron.emails.async
            # Mailer.delay.new_item_like(activity.item, activity.performed_by)
          # else
            # Mailer.new_item_like(activity.item, activity.performed_by).deliver
          # end
        # end
      # end  
    # end
    
    after_destroy do |activity|
      activity.decrement_users_activities_count
    end
    after_destroy do |activity|
      activity.item.decrement!(:comments_count) if activity.comment? && activity.item_id.present?
    end
  #-----

  #-Scopes-----
    scope :likes, where(kind: LIKE)
    scope :leaves, where(kind: LEAVE)
    scope :comments, where(kind: COMMENT)
    scope :new_items, where(kind: CREATE_ITEM)
    
    scope :visible, where(:visible => true)
  #-----

  %w(like comment create_item accept_invitation send_invitation).each do |activity_kind|
    define_method("#{activity_kind}?") { kind == activity_kind }
  end
  
  def read_by?(user, visit = nil)
    return true if self.user_id == user.id
    
    visit ||= user.visits.where(:message_id => self.message_id).first
    
    last_visit_date = visit.last_visit
    last_visit_date.present? ? (last_visit_date.utc.to_i >= self.created_at.utc.to_i) : false
  end

  #-------#
  protected
  #-------#

  def update_message_last_activity_time
    self.message.update_attribute(:last_activity_at, self.created_at)
  end
  
  def increment_users_activities_count
    Visit.where(:message_id => self.message_id).where("user_id <> #{ self.performed_by_id }").each do |visit|
      visit.increment!(:activities_count)
    end
  end
  def decrement_users_activities_count
    Visit.where(:message_id => self.message_id).where("last_visit < '#{ self.created_at }'").each do |visit|
      visit.decrement!(:activities_count)
    end
  end
end