class Scraper < ActiveRecord::Base
  attr_accessible :brand_name, :desc, :domain, :image_url, :name, :price, :title, :default_brand
  has_many :missed_url

  def self.decoded_url(url)
    URI.decode(url)
  end
  def self.encoded_url(url)
    URI.encode( self.decoded_url(url) )
  end
  
  def scrap(original_url)
    url = self.class.decoded_url( original_url )
    
    domain = self.class.get_domain(url)
        
    selectors = {
      :title => [self.title],
      :desc => [self.desc],
      :brand_name => [self.brand_name],
      :price => [self.price],
      :image_url => [self.image_url],
      :default_brand => [self.default_brand],
      :skip_diffbot => true
    }
    selectors[:domain] = domain
    
    self.class.scrap_with_selectors(url, selectors)
  end
  
  def self.scrap(original_url)
    url = self.decoded_url( original_url )
    
    begin
      @scraper = self.get_scraper_for_url(url)
      
      if @scraper
        @scraper.scrap(url)
      else
        domain = get_domain(url)
        
        selectors = { }
        self.update_missed_url(domain, url)
        selectors[:domain] = domain
        
        self.scrap_with_selectors(url, selectors)
      end
    rescue Exception => e
      Airbrake.notify_or_ignore(e, {
        :parameters => {
          :original_url => original_url,
          :url => url
        }
      })
      
      nil
    end
  end

  def self.diffbot(original_url)
    url = self.encoded_url(original_url)
    
    diffbot_token = "61a7da539d9a28e7caa25942c6e41719"
    agent = Mechanize.new
    page = agent.get('http://api.diffbot.com/v2/product?token=' + diffbot_token + "&url=" + url).body
    JSON.parse(page)['products'][0]
  end

  # TODO: refactor this beast
  def self.scrap_with_selectors(original_url, opts = {})
    url = self.encoded_url(original_url)
    
    t1 = Thread.new { self.get_document( self.decoded_url(url) ) }
    t2 = Thread.new {
      if configatron.scraper.diffbot_enabled && !opts[:skip_diffbot]
        begin
          self.diffbot(url)
        rescue Exception => e
          Airbrake.notify_or_ignore(e, {
            parameters: {
              url: url
            }
          })
          
          nil
        end
      else
        nil
      end
    }
    
    begin
      doc = t1.value
    rescue Exception => e
      Airbrake.notify_or_ignore(e, :parameters => {
        :problem => 'Failed while grabbing url using Mechanize',
        :url => url,
        :opts => opts
      })
    end
    
    # unless doc
      # t2.kill
      # return
    # end

    diffbot_data = t2.value

    # Generic selectors (maybe refactor these to be in the data layer for easier editing?)
    selectors = opts
    generic_selectors = {
      :title => ["//*[@itemprop='name']", "//meta[@property='og:title']", "//meta[@name='title']", "//title"],
      :desc => [ "//*[@itemprop='description']", "//meta[@property='og:description']", "//meta[@name='description']"],
      :brand_name => ["//*[@itemprop='brand']"],
      :price => ["//*[@itemprop='price']"],
      :image_url => ["//meta[@property='og:image']", "//*[@itemprop='image']", "//meta[@name='image']", "//img"]
    }

    # TODO: Clean-up the following line
    selectors.each { |label, _| selectors[label].is_a?(Array) && selectors[label].reject!(&:blank?) }
    selectors[:domain] = Array.wrap(opts[:domain]).first

    data = {}

    # Nasty hack for gap.com websites where the info is hidden in JS
    if url.include? "gap.com"
      data = self.get_gap_data(url)
    end

    if doc
      [:title, :desc, :brand_name, :price].each do |element|
        data[element] = self.extract_content(self.search_for_elements(doc, selectors[element]))
      end
    end

    data[:brand_name] ||= selectors[:default_brand].try(:first)
    data[:brand_name] ||= selectors[:domain]
    
    if diffbot_data
      data[:brand_name] ||= diffbot_data['brand']
      data[:title] ||= diffbot_data['title']
      data[:desc] ||= diffbot_data['description']
      data[:price] ||= diffbot_data['offerPrice']
    end

    if doc
      [:title, :desc, :brand_name, :price].each do |element|
        data[element] ||= self.extract_content(self.search_for_elements(doc, generic_selectors[element]))
      end
    end

    data[:brand] = data.delete(:brand_name)
    data[:description] = data.delete(:desc)

    data[:images_urls] ||= []
    if doc
      data[:images_urls] += self.search_for_elements(doc, selectors[:image_url], true).map{|a|
        Scraper.extract_content(a)
      }
    end
    data[:images_urls] = data[:images_urls].select {|i| !i.blank? }
    if data[:images_urls].empty? && diffbot_data && diffbot_data['media']
      data[:images_urls] = diffbot_data['media'].select{|i| i["type"] == "image"}.map{|i| i["link"]}
    end
    if data[:images_urls].empty? && doc
      data[:images_urls] = self.search_for_elements(doc, generic_selectors[:image_url], true).map{|a|
        Scraper.extract_content(a)
      }
    end
    data[:images_urls].reject! {|i| !i}
    data[:images_urls].map! {|i| self.absolute_url(url, i) }
    data[:prefered_image_url] = data[:images_urls].first
    data[:url] = self.decoded_url(url)
    
    begin
      data.each_value {|v| v.strip! if v.is_a?(String) }
    rescue Exception => e
      Airbrake.notify_or_ignore(e)
    end
    
    data
  end

  def self.search_for_elements(doc, selectors, multiple = false)
    elements = []
    selectors ||= []
    selectors.each do |selector|
      elements += doc.search(selector)
    end
    multiple ? elements : elements[0]
  end

  def self.get_document(url)
    agent = Mechanize.new
    agent.user_agent_alias = 'Mac Safari'
    agent.verify_mode = OpenSSL::SSL::VERIFY_NONE
    begin
      agent.get(url.index(/https?:\/\//) == 0 ? url : "http://#{url}").root
    rescue SocketError => e
      return false
    rescue ArgumentError
      return false # Or better, raise an exception
    end
  end

  def self.extract_content(node)
    return nil unless node
    if node.name =='meta'
      text = node.attributes['content'].value
    elsif node.name =='img' && node.attributes['src']
      text = node.attributes['src'].value
    else
      text = node.text
    end
    text
  end

  # TODO: get_scraper_for_url should use scraper_matches_domain (for consistency)
  def self.get_scraper_for_url(url)
    domain = get_domain(url)
    self.find_by_domain(domain)
  end

  def self.scraper_matches_domain(url, domain)
    get_domain(url) == domain
  end

  #-----#
  private
  #-----#
  
  #get the domian without www
  def self.get_domain(original_url)
    decoded_url = self.decoded_url( original_url )
    encoded_url = self.encoded_url( original_url )
    
    if URI.parse(encoded_url).scheme.nil?
      decoded_url = "http://#{ decoded_url }"
      encoded_url = self.encoded_url( decoded_url )
    end
    
    host = URI.parse(encoded_url).host.downcase
    host.start_with?('www.') ? host[4..-1] : host
  end

  def self.absolute_url(page_url, url)
    url = url.strip
    
    return url if url.starts_with?('data:')
    
    url = URI.encode(url)
    response = URI.parse( page_url ).merge( URI.parse(url) ).to_s
    
    URI.decode(response)
  end

  # def self.relative_to_absolute(relative, url)
    # return unless relative
    # return relative if relative.index(/https?:\/\//) == 0
# 
    # if relative[0] == '/'
      # uri = URI.parse(url)
      # "#{uri.scheme}://#{uri.host}#{relative}"
    # else
      # "#{url}#{relative}"
    # end
  # end

  def self.update_missed_url(domain,url,scraper=nil)
   @missed_domain = MissedDomains.find_by_domain(domain)
    if @missed_domain
      @missed_domain.count += 1
      @missed_domain.save
    else
      @missed_domain = MissedDomains.new()
      @missed_domain.domain = domain
      @missed_domain.count = 1
      @missed_domain.save
    end
    @missed_url = MissedUrl.new()
    @missed_url.missed_domains = @missed_domain
    @missed_url.url = url
    @missed_url.scraper = scraper
    @missed_url.save
  end

  def self.get_gap_data(url)
    data = {}
    m = url.match /pid=(\d+)/
    product_id = m[1]
    domain = self.get_domain(url)
    product_url = domain + "/browse/productData.do?pid=" + product_id
    product = self.get_document(product_url)
    m = product.to_str.match /P01.+?'(.+?)'/
    data[:images_urls] = [m[1]]
    data
  end
end
