Tisket::Application.routes.draw do
  match "/delayed_job" => DelayedJobWeb, :anchor => false
  DelayedJobWeb.use Rack::Auth::Basic do |username, password|
    username == configatron.delayed_job.username && password == configatron.delayed_job.password
  end
  
  resource :delay, only: :show, controller: 'delay'
  
  resource :search, only: :show, controller: 'search'
  
  get '/profile' => 'profiles#show', as: :my_profile
  resources :profiles, only: :show
  
  resources :scrapers, only: :index
  
  resources :urls, :only => [] do
    collection do
      post :scrape
    end
  end
  
  resources :search_users, only: [:index, :create] do
    collection do
      get :tokens
    end
  end
  
  resources :users, only: [:edit, :update] do
    member do
      put 'update_avatar'
      put 'update_notification_settings'
    end
  end

  resource :explore, controller: 'explore'

  namespace :messages, path: 'collections' do
    resource :widget, :controller => 'widget'
  end
  resources :messages, path: 'collections', only: [:show, :new, :create, :edit, :update] do
    resources :items, only: :create, controller: 'messages/items'
    
    resource :leave, :only => [:show, :create], :controller => 'messages/leave'
    
    resource :old_widget, :controller => 'messages/old_widget' do
      member do
        get :test
      end
    end
    
    resource :personalization, controller: 'messages/personalization', only: [:show, :create]
    
    collection do
      get 'dashboard'
      get 'open'
      get 'public', :action => 'open', :public => true
      get 'closed'
    end

    member do
      post 'change_title'
      post 'change_description'
      get 'new_invitation'
      put 'send_invitation'
      get 'accept_invitation'
      get 'change_featured_item'
      delete 'delete_item/:item' => 'messages#delete_item', :as => 'delete_item'
      
      put 'like_item/:item' => 'messages#like_item', :as => 'like_item' 
      put 'unlike_item/:item' => 'messages#unlike_item', :as => 'unlike_item'
      put 'leave_item/:item' => 'messages#leave_item', :as => 'leave_item' 
      put 'unleave_item/:item' => 'messages#unleave_item', :as => 'unleave_item' 
      put 'save_item/:item' => 'messages#save_item', :as => 'save_item'
      
      post 'add_item'
      post 'add_comment'
      delete 'delete_comment'
      get 'select_items'
      delete 'close'
      put 'update_last_visit_time'
      get 'activities'
      get 'cycle/:current_item' => 'messages#cycle_items', :as => 'cycle_iems'
      delete 'remove_user'
    end
    
    get 'widget' => 'messages#widget', version: 1, as: :widget_v1
    get 'v2/widget' => 'messages#widget', version: 2, as: :widget_v2
  end

  resources :items, only: [:show, :update] do
    resource :copy_or_move, controller: 'items/copy_or_move', only: :create
    
    resources :images, only: :index, controller: 'items/images'
    resource :image, only: :create, controller: 'items/image'
    
    resources :comments, only: :index, controller: 'items/comments'
    
    resources :clicks, only: :create, controller: 'items/clicks'
    
    resource :save, only: :create, controller: 'items/save'
    
    collection do
      get 'scrap'
      get 'scraping_run'
      get 'scraping'
      post 'add_to_existing_messages'
      post 'add_to_new_message'
      post 'post_to_existing_messages'
      post 'post_to_new_message'
    end
  end
  
  resources :activities, only: [] do
    member do
      post :like
      post :unlike
      
      get :likes
    end
  end
  
  resources :errors, :only => [] do
    collection do
      get :error_404
      get :error_500
    end
  end
  
  # get "/welcome" => "home/welcome", as: :welcome
  get "/info" => "home#info", as: :info 
  get "/about" => "home#about", as: :about
  get "/terms" => "home#terms", as: :terms
  get "/privacy" => "home#privacy", as: :privacy

  resource :bookmarklet, :controller => 'bookmarklet'
  resource :bookmarklet2, :controller => 'bookmarklet2'

  devise_for(:users, path: 'accounts', controllers: {
    :omniauth_callbacks => 'users/omniauth_callbacks',
    :invitations => 'users/invitations',
    :passwords => 'users/passwords',
    :registrations => 'users/registrations',
    :sessions => 'users/sessions'
  })
  
  # devise_scope :user do
    # get '/' => 'users/invitations#new'
  # end  
  #root to: "users/invitations#new"
  root to: "home#show" 

  devise_for :admin_users, ActiveAdmin::Devise.config
  ActiveAdmin.routes(self)

  # The priority is based upon order of creation:
  # first created -> highest priority.

  # Sample of regular route:
  #   match 'products/:id' => 'catalog#view'
  # Keep in mind you can assign values other than :controller and :action

  # Sample of named route:
  #   match 'products/:id/purchase' => 'catalog#purchase', :as => :purchase
  # This route can be invoked with purchase_url(:id => product.id)

  # Sample resource route (maps HTTP verbs to controller actions automatically):
  #   resources :products

  # Sample resource route with options:
    # resources :products do
    #   member do
    #     get 'short'
    #     post 'toggle'
    #   end
  
    #   collection do
    #     get 'sold'
    #   end
    # end

  # Sample resource route with sub-resources:
  #   resources :products do
  #     resources :comments, :sales
  #     resource :seller
  #   end

  # Sample resource route with more complex sub-resources
  #   resources :products do
  #     resources :comments
  #     resources :sales do
  #       get 'recent', :on => :collection
  #     end
  #   end

  # Sample resource route within a namespace:
  #   namespace :admin do
  #     # Directs /admin/products/* to Admin::ProductsController
  #     # (app/controllers/admin/products_controller.rb)
  #     resources :products
  #   end

  # You can have the root of your site routed with "root"
  # just remember to delete public/index.html.
  # root :to => 'welcome#index'

  # See how all your routes lay out with "rake routes"

  # This is a legacy wild controller route that's not recommended for RESTful applications.
  # Note: This route will make all actions in every controller accessible via GET requests.
  # match ':controller(/:action(/:id))(.:format)'
end