$(document).ready(function() {
  var errors = {
    problemOccurred: "We had some trouble pulling the product information. Email us at contact@tisket.com with the link you are trying to add, and we'll be happy to help out!",
    notLoggedIn: "Please login on tisket to grap your collections",
    collectionNameMissing: "Please write a collection name"
  };

  var notificationHandler = function(){
    var that = {};
    
    var notification = $('.js-notification');
    
    var showError = function(error){
      notification.removeClass('success');
      notification.addClass('error');
      notification.find('.js-message').html( errors.problemOccurred );
      notification.show();
    };
    that.showError = showError;
    
    var showMessage = function(message){
      notification.removeClass('error');
      notification.addClass('success');
      notification.find('.js-message').html( message );
      notification.show();
    };
    that.showMessage = showMessage;
    
    var hide = function(){
      notification.hide();
    };
    that.hide = hide;
    
    return that;
  }();

  var collectionsDDL = $('#collection_id');

  // get a param by name from page url (window.location) in javascript
  var getParameterByName = function(name) {
    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
        results = regex.exec(location.search);
    return results == null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
  };
  
  var addItemToCollection = function(collectionSource){
    // if the collectionSource is ddl it will use the val of collection_id element
    // else it will use the string sored in the variable collectionSource itself
    collectionId = (collectionSource == 'ddl') ? collectionsDDL.val() : collectionSource ;
    // make sure that the collectionId is selected
    if (collectionId == ''){
      alert('Please select a collection');
    }else{
      // grap the data
      data = {
        item : {
          title: $('#item_title').text(),
          image_url: $('#item_image').prop('src'),
          url: $('#item_url').val()
        }
      }

      // make a post request to tisket rails application
      $.ajax({
        type: "POST",
        url: settings.hostName() + 'collections/' + collectionId + '/add_item.extchr',
        data: data,
        beforeSend: function(){
          notificationHandler.hide();
          
          $('#add_item_btn').prop("disabled",true);
          $('#add_item_btn').nextAll('.js-spinner').show();
        },
        success: function(result){
          notificationHandler.showMessage( result );
          
          setTimeout(function(){
            closeOverlay();
          }, 1500);
        },
        error: function(){
          notificationHandler.showError( errors.problemOccurred );
        },
        complete: function(xhr, textStatus) {
          $('#add_item_btn').prop("disabled",false);
          $('#add_item_btn').nextAll('.js-spinner').hide();
        }
      });
    }
  };

  // Create the collection then call the function addItemToCollection
  var createCollectionThenAddItem = function(){
    if ($('#collection_subject').val() == ''){
      notificationHandler.showError( errors.collectionNameMissing );
    }else{
      // grap the data
      data = {
        message: {
          subject: $('#collection_subject').val()
        }
      }

      // make a post request to tisket rails application
      $.ajax({
        type: "POST",
        url: settings.hostName() + 'collections.extchr',
        data: data,
        beforeSend: function(){
          notificationHandler.hide();
          
          $('#add_item_btn').prop("disabled",true);
          $('#add_item_btn').nextAll('.js-spinner').show();
        },
        success: function(result){
          addItemToCollection(result);
        },
        error: function(){
          $('#add_item_btn').prop("disabled",false);
          $('#add_item_btn').nextAll('.js-spinner').hide();
          
          notificationHandler.showError( errors.problemOccurred );
        }
      });
    }
  };
  
  var closeOverlay = function(){
    parent.postMessage('closeIframe', '*');
  };
  
  // Populate some values from url
  $('#item_title').html( unescape(getParameterByName('title')) );
  var imageUrl = getParameterByName('image_url');
  if( imageUrl.length > 0 )
    $('#item_image').attr('src', unescape(imageUrl));
  else{
    $('#item_image').remove();
    $('.js-overlay').addClass('withoutImg');
  }
  $('#item_url').val( unescape(getParameterByName('url')) );
  
  // Load open collections of loggedIn User
  $.ajax({
    url: settings.hostName() + 'collections/open.extchr',
    success: function(result){
      result = jQuery.parseJSON(result);
      for (var i = 0; i < result.length; i++) {
        option = $('<option value="'+result[i].id+'">'+result[i].subject+'</option>');
        option.appendTo(collectionsDDL);
      };
    },
    error: function(xhr, textStatus) {
      if(xhr.status == 401){
        window.location.href = ('/html/login.html' + window.location.search);
      }
      else{
        closeOverlay();
        
        alert(errors.problemOccurred);
      }
    }
  });

  $('#add_item_btn').click(function(){
    var selectedRadio = $('input:radio').filter(':checked');
    
    if(selectedRadio.attr('id') == 'radio_collection_id')
      addItemToCollection('ddl');
    else if(selectedRadio.attr('id') == 'radio_collection_subject')
      createCollectionThenAddItem();
  });

  // close overlay when Esc is clicked
  $(document).keyup(function(e) {
    if (e.keyCode == 27) {
      closeOverlay();
    }
  });

  $('.js-closeTrigger').click(function(){
    closeOverlay();
  });

  $('.js-newCollectionOption').click(function(){
    var field = $('.js-newCollectionSubject');
    
    $('.js-existingCollectionOption').attr('checked', false);
    
    if( field.val().length == 0 )
      field.val( (new Date()).format("mmmm d, yyyy") );
  });
  $('.js-newCollectionSubject').focus(function(){
    var field = $(this);
    
    $('.js-newCollectionOption').attr('checked', true);
    $('.js-existingCollectionOption').attr('checked', false);
    
    if( field.val().length == 0 )
      field.val( (new Date()).format("mmmm d, yyyy") );
  });
});