module GoogleAnalytics
  class Importer
    def client
      return @client unless @client.nil?
      
      # Inspired by https://gist.github.com/3166610
      require 'google/api_client'
      require 'date'
      
      api_version = 'v3'
      cached_api_file = "analytics-#{api_version}.cache"
      
      # Update these to match your own apps credentials
      service_account_email = '91481402305-7h6q2akcoa9hccsqjplqtivpqghi9mcl@developer.gserviceaccount.com' # Email of service account
      key_file = '5da5d2fef0d560f4943dc9d9bef6c6ac7c8c9986-privatekey.p12' # File containing your private key
      key_secret = 'notasecret' # Password to unlock private key
      @profileID = configatron.google_analytics.profile_id # Analytics profile ID.
      
      client = Google::APIClient.new(
        :application_name => 'Ruby Service Accounts sample',
        :application_version => '1.0.0')
      
      # Load our credentials for the service account
      key = Google::APIClient::KeyUtils.load_from_pkcs12(
        "#{ Rails.root }/lib/google_analytics/#{ key_file }", key_secret
      )
      client.authorization = Signet::OAuth2::Client.new(
        :token_credential_uri => 'https://accounts.google.com/o/oauth2/token',
        :audience => 'https://accounts.google.com/o/oauth2/token',
        :scope => 'https://www.googleapis.com/auth/analytics.readonly',
        :issuer => service_account_email,
        :signing_key => key)
      
      # Request a token for our service account
      client.authorization.fetch_access_token!
      
      analytics = nil
      # Load cached discovered API, if it exists. This prevents retrieving the
      # discovery document on every run, saving a round-trip to the discovery service.
      if File.exists?(cached_api_file)
        File.open(cached_api_file) do |file|
          analytics = Marshal.load(file)
        end
      else
        analytics = client.discovered_api('analytics', api_version)
        File.open(cached_api_file, 'w') do |file|
          Marshal.dump(analytics, file)
        end
      end
      @analytics = analytics
      
      @client = client
      
      @client
    end
    
    def run(options = {})
      start_date = options[:start_date] || DateTime.now.prev_month.strftime("%Y-%m-%d")
      end_date = options[:end_date] || DateTime.now.strftime("%Y-%m-%d")
      
      response = self.client.execute(:api_method => @analytics.data.ga.get, :parameters => { 
        'ids' => "ga:" + @profileID, 
        'start-date' => start_date,
        'end-date' => end_date,
        'dimensions' => "ga:eventAction,ga:eventLabel",
        'metrics' => "ga:eventValue",
        'filters' => 'ga:eventCategory==users',
        'max-results' => 10000
      })
      
      if response.error?
        exception = Exception.new(response.error_message)
        Airbrake.notify_or_ignore(exception)
        
        raise(exception)
      end
      
      result = Hash.new {|h,k| h[k] = {} }
      response.data.rows.each do |entry|
        result[ entry[0] ][ entry[1] ] = 1
      end
      
      final_result = Hash.new {|h,k| h[k] = 0 }
      result.each_pair do |action_name, labels_hash|
        final_result[action_name] = labels_hash.keys.length
      end
      
      final_result
    end
  end
end